open Jabber
open Jabbersm
open Yaxpodom

type credentials =
    Plaintext of string*string
  | Digest of string*string*string

class type auth_machine =
object
  inherit state_machine

  method start : (unit -> unit)
  method reject_reason : (unit -> string)
end

class auth_machine_impl creds resource write =
object (self)
  val mutable external_state = Start
  val mutable kont = None
  val mutable error = ""

  method private mk_packet children =
    packet_of_ele (Element(QName("","iq","jabber:client"),
			   ref [ Att(QName("","type",""), ref "set");
				 Att(QName("","id",""), ref "do_auth") ],
			   ref [ Subele(
				   Element(
				     QName("","query","jabber:iq:auth"),
				     ref [ Att(QName("","xmlns",""), ref "jabber:iq:auth") ],
				     ref children)) ]))

  method private mk_plaintext_children username password =
    [ Subele(Element(QName("","username","jabber:iq:auth"),
		     ref [],
		     ref [ Text(username) ]));
      Subele(Element(QName("","password","jabber:iq:auth"),
		     ref [],
		     ref [ Text(password) ])) ]

  method private mk_digest_children username password stream_id =
    let hashval = Cryptokit.hash_string (Cryptokit.Hash.sha1 ()) (stream_id ^ password) in
    let hashtxt = Cryptokit.transform_string (Cryptokit.Hexa.encode ()) hashval in
      [ Subele(Element(QName("","username","jabber:iq:auth"),
		       ref [],
		       ref [ Text(username) ]));
	Subele(Element(QName("","digest","jabber:iq:auth"),
		       ref [],
		       ref [ Text(hashtxt) ])) ]
      

  method start () =
    (write
       (self#mk_packet
	  (List.concat
	     [
	       (match creds with
		    Plaintext(u,p) -> self#mk_plaintext_children u p
		  | Digest(u,p,i) -> self#mk_digest_children u p i);
	       [ Subele(Element(QName("","resource","jabber:iq:auth"),
				ref [],
				ref[ Text(resource) ])) ]
	     ])));
     external_state <- Intermediate;
     kont <- Some
     (fun p ->
	(match p with
	     IQ(iq) ->
	       if iq#get_id () = Some("do_auth") then
		 (match iq#get_type () with
		      Some Result -> external_state <- Accept
		    | Some IQError -> 
			external_state <- Reject;
			(try
			   let (code,desc) = iq#get_error () in
			     error <- desc
			 with
			     _ -> error <- "authentication failure")
		    | _ -> external_state <- Reject; error <- "unrecognized reply")
	       else ()
	   | _ -> ()))

  method get_state () =
    external_state

  method add_packet p =
    (match kont with
	 Some(k) -> k p
       | None -> failwith "unexpected add_packet")

  method reject_reason () =
    error
end

let mk_auth_machine creds resource write =
  (new auth_machine_impl creds resource write :> auth_machine)
