(** jabber:iq:auth authentication state machine. It is not necessary to use this directly if you use the [Jabber_sync] module. *)

open Jabber
open Jabbersm

(** Authentication credentials.*)
type credentials =
    Plaintext of string*string          (** (username,password) *)
  | Digest of string*string*string      (** (username,password,streamID) *)

(** The type of the jabber:iq:auth authenticator state machine. *)
class type auth_machine =
object
  inherit state_machine

  (** Start the state machine (i.e., transmit auth packet) *)
  method start : (unit -> unit)

  (** in the event of an authentication failure (state = Reject), retrieve the reason therefore. *)
  method reject_reason : (unit -> string)
end

(** [mk_auth_machine creds resource write_packet] creates a new authenticator given the credentials, resource, and a procedure that writes a packet back to the server. *)
val mk_auth_machine : (credentials -> string -> (packet -> unit) -> auth_machine)
