open Yaxpodom

exception Bad_packet

class type packet_base =
object
  method get_from : (unit -> string option)
  method set_from : (string option -> unit)

  method get_to : (unit -> string option)
  method set_to : (string option -> unit)

  method get_id : (unit -> string option)
  method set_id : (string option -> unit)

  method get_children : (unit -> content list)
  method set_children : (content list -> unit)

  method get_error : (unit -> txt*txt)


  method to_ele : (unit -> ele)
end

class packet_base_impl my_ele =
object (self)
  method private get_att name =
    try
      Some(ele_get_att' my_ele (QName("",name,"")))
    with
	Not_found -> None

  method private set_att name value =
    (match value with
	 Some(v) -> ele_set_att my_ele (QName("",name,"")) v
       | None -> ele_remove_att' my_ele (QName("",name,"")))

  method get_from () =
    self#get_att "from"
	
  method set_from value =
    self#set_att "from" value

  method get_to () =
    self#get_att "to"

  method set_to value =
    self#set_att "to" value

  method get_id () =
    self#get_att "id"

  method set_id value =
    self#set_att "id" value

  method get_children () =
    ele_children my_ele

  method set_children children =
    set_ele_children my_ele children

  method get_error () =
    let rec helper children =
      (match children with
	   Subele(Element(QName(_,"error","jabber:client"),_,_) as error_ele) :: rest ->
	     let code = try ele_get_att' error_ele (QName("","code","")) with Not_found -> "" in
	     let desc = ele_inner_text error_ele in
	       (code,desc)
	 | _ :: rest -> helper rest
	 | [] -> raise Not_found) in
      helper (ele_children my_ele)

  method to_ele () =
    my_ele
end

type iq_type =
    Get
  | Set
  | Result
  | IQError

class type iq =
object
  inherit packet_base

  method get_type : (unit -> iq_type option)
  method set_type : (iq_type option -> unit)
    
  method get_first_query : (unit -> ele)
  method get_first_query_xmlns : (unit -> string)
end

class iq_impl my_ele =
object (self)
  inherit packet_base_impl my_ele

  method get_type () =
    try
      (match ele_get_att' my_ele (QName("","type","")) with
	  "get" -> Some Get
	| "set" -> Some Set
	| "result" -> Some Result
	| "error" -> Some IQError
	| _ -> raise Bad_packet)
    with
	Not_found -> None

  method set_type value =
    match value with
	Some(v) ->
	  ele_set_att my_ele (QName("","type",""))
	  (match v with
	       Set -> "set"
	     | Get -> "get"
	     | Result -> "result"
	     | IQError -> "error")
      | None -> ele_remove_att' my_ele (QName("","type",""))

  method get_first_query () =
    match (ele_children my_ele) with
	Subele(
	  Element(
	    QName(_,"query",_), _, _) as it) :: rest -> it
      | _ -> raise Not_found

  method get_first_query_xmlns () =
    match self#get_first_query () with
	Element(
	  QName(_,"query",nsURI),_,_) -> nsURI
      | _ -> assert false
end
      
let mk_iq prefix =
  (new iq_impl (Element(QName(prefix,"iq","jabber:client"),ref [],ref [])) :> iq)

(** presence "type" attribute values *)
type presence_type =
    Unavailable
  | Subscribe
  | Subscribed
  | Unsubscribe
  | Unsubscribed
  | Probe
  | PresError

(** the type of presence packet objects *)
class type presence =
object
  inherit packet_base

  method get_type : (unit -> presence_type option)
  method set_type : (presence_type option -> unit)

  method get_show : (unit -> string option)
  method get_status : (unit -> string option)
  method get_priority : (unit -> string option)
end

let rec get_child_innertext name children =
  match children with
      [] -> None
    | Subele(
	Element(
	  QName(_,name',"jabber:client"),
	  _,_) as an_ele) :: rest when name = name' -> Some(ele_inner_text an_ele)
    | _ :: rest -> get_child_innertext name rest

class presence_impl (my_ele) =
object
  inherit packet_base_impl my_ele

  method get_type () =
    try
      (match ele_get_att' my_ele (QName("","type","")) with
	  "unavailable" -> Some Unavailable
	| "subscribe" -> Some Subscribe
	| "subscribed" -> Some Subscribed
	| "unsubscribe" -> Some Unsubscribe
	| "unsubscribed" -> Some Unsubscribed
	| "probe" -> Some Probe
	| "error" -> Some PresError
	| _ -> raise Bad_packet)
    with
	Not_found -> None

  method set_type value =
    match value with
	Some(v) ->
	  ele_set_att my_ele (QName("","type",""))
	  (match v with
	       Unavailable -> "unavailable"
	     | Subscribe -> "subscribe"
	     | Subscribed -> "subscribed"
	     | Unsubscribe -> "unsubscribe"
	     | Unsubscribed -> "unsubscribed"
	     | Probe -> "probe"
	     | PresError -> "error")
      | None -> ele_remove_att' my_ele (QName("","type",""))

  method get_show () =
    get_child_innertext "show" (ele_children my_ele)

  method get_status () =
    get_child_innertext "status" (ele_children my_ele)

  method get_priority () =
    get_child_innertext "priority" (ele_children my_ele)
end

let mk_presence prefix =
  (new presence_impl (Element(QName(prefix,"presence","jabber:client"),ref [], ref [])) :> presence)

type message_type =
    Normal
  | Chat
  | Groupchat
  | Headline
  | Error

class type message =
object
  inherit packet_base

  method get_type : (unit -> message_type option)
  method set_type : (message_type option -> unit)

  method get_body : (unit -> string option)
  method get_subject : (unit -> string option)
  method get_thread : (unit -> string option)
end

class message_impl my_ele = 
object
  inherit packet_base_impl my_ele

  method get_type () =
    try
      (match ele_get_att' my_ele (QName("","type","")) with
	   "normal" -> Some Normal
	 | "chat" -> Some Chat
	 | "groupchat" -> Some Groupchat
	 | "headline" -> Some Headline
	 | "error" -> Some Error
	 | _ -> raise Bad_packet)
    with
	Not_found -> None

  method set_type value =
    match value with
	Some(v) ->
	  ele_set_att my_ele (QName("","type",""))
	  (match v with
	       Normal -> "normal"
	     | Chat -> "chat"
	     | Groupchat -> "groupchat"
	     | Headline -> "headline"
	     | Error -> "error")
      | None -> ele_remove_att' my_ele (QName("","type",""))

  method get_body () =
    get_child_innertext "body" (ele_children my_ele)    

  method get_thread () =
    get_child_innertext "thread" (ele_children my_ele)

  method get_subject () =
    get_child_innertext "subject" (ele_children my_ele)
end

let mk_message prefix =
  ((new message_impl (Element(QName(prefix,"message","jabber:client"),ref [],ref []))) :> message)

type packet =
    IQ of iq
  | Presence of presence
  | Message of message

let packet_of_ele the_ele =
  (match ele_name the_ele with
       QName(_,"iq","jabber:client") -> IQ(new iq_impl the_ele :> iq)
     | QName(_,"presence","jabber:client") -> Presence(new presence_impl the_ele :> presence)
     | QName(_,"message","jabber:client") -> Message(new message_impl the_ele :> message)
     | _ -> raise Bad_packet)

let packet_base_of_packet p =
  (match p with
       IQ(iq) -> (iq :> packet_base)
     | Presence(p) -> (p :> packet_base)
     | Message(m) -> (m :> packet_base))

let packet_to p =
  (packet_base_of_packet p)#get_to ()

let packet_from p =
  (packet_base_of_packet p)#get_from ()

let packet_id p =
  (packet_base_of_packet p)#get_id ()
