(** An abstraction for Jabber Identifiers (JIDs). *)

(** The type of JIDs. *)
type t

(** Construct a JID from (optional) node, domain, and (optional) resource. *)
val mk_jid : (string option -> string -> string option -> t)

(** Bad JID format. *)
exception Bad_jid

(** Parses a JID.

@raise Bad_jid if the JID is invalid. *)
val of_string : (string -> t)

(** Emits a JID. *)
val to_string : (t -> string)

val node : (t -> string option)
val domain : (t -> string)
val resource : (t -> string option)
(** JID accessors *)

val set_node : (t -> string option -> unit)
val set_domain : (t -> string -> unit)
val set_resource : (t -> string option -> unit)
(** JID mutators *)

(** Returns a new JID without the resource. *)
val strip_resource : (t -> t)
